@@ -1,8 +1,14 @@
 """account_settings.py
 
 Access class for account settings, with cache
+  Each account is associated with an entry in the Settings model; this
+  class provides access to these key-value pairs as read-only
+  attributes. Because these functions are called frequently, the db
+  entry is loaded into memory for a configurable expiration period
+  (config.REDIS_TTL).
 
 created 13-may-2019 by richb@instantlinux.net
+
 """
 
 from collections import namedtuple
@@ -11,12 +17,20 @@ import logging
 from sqlalchemy.orm.exc import NoResultFound
 
 from . import utils
+from .service_config import ServiceConfig
 
 SETTINGS = {}
 
 
 class AccountSettings(object):
-    def __init__(self, account_id, config, models, db_session=None, uid=None):
+    """Account Settings - converts db record to object attributes.
+
+    Args:
+      account_id (str): ID in database of a user's account
+      db_session (obj): a session connected to datbase
+      uid (str): User ID
+    """
+    def __init__(self, account_id, db_session=None, uid=None):
         """Cache per-account settings and convert to attributes"""
 
         def _convert(attrs):
@@ -24,6 +38,7 @@ class AccountSettings(object):
 
         if account_id not in SETTINGS or (
                 utils.utcnow() > SETTINGS[account_id]['expires']):
+            models = ServiceConfig().models
             try:
                 if account_id:
                     account = db_session.query(models.Account).filter_by(
@@ -44,9 +59,9 @@ class AccountSettings(object):
                         name='default').one().id
             except NoResultFound:
                 category_id = account.settings.default_cat_id
+            config = ServiceConfig().config
             SETTINGS[account_id] = dict(
-                expires=utils.utcnow() + timedelta(
-                    seconds=config.REDIS_TTL),
+                expires=utils.utcnow() + timedelta(seconds=config.REDIS_TTL),
                 settings=dict(
                     record.as_dict(), **dict(
                         category_id=category_id,
