@@ -1,6 +1,7 @@
 """confirmation.py
 
-Handle the request and confirm actions for a contact.
+Confirmation
+  Handle the request and confirm actions for a contact.
 
 created 4-apr-2020 by richb@instantlinux.net
 """
@@ -12,13 +13,23 @@ import random
 from sqlalchemy.orm.exc import NoResultFound
 import string
 
-from ..constants import i18n
+from ..const import i18n
+from ..service_config import ServiceConfig
 
 
 class Confirmation:
+    """Functions for confirming ownership of online email/contact info;
+    class attributes are passed through the service config.
 
-    def __init__(self, config, models):
-        self.models = models
+    Attributes:
+      token_salt (str): salt value for serializer token
+      token_secret (str): a secret to validate upon confirmation
+      token_timeout (int): seconds until expiration
+    """
+
+    def __init__(self):
+        config = ServiceConfig().config
+        self.models = ServiceConfig().models
         self.token_salt = config.TOKEN_SALT
         self.token_secret = config.TOKEN_SECRET
         self.token_timeout = config.TOKEN_TIMEOUT
@@ -27,6 +38,17 @@ class Confirmation:
                 func_send=None):
         """Generate a confirmation token valid for a given period,
         and send it to the contact identified by id
+
+        Args:
+          id (str): record ID in contacts table of database
+          ttl (int): how many seconds before token expires
+          message (str): key in i18n dict for contact-add message
+          func_send (function): name of function for sending message
+
+        Returns:
+          tuple:
+            first item is dict with token and contact-ID, second is
+            the http response code
         """
 
         ttl = self.token_timeout if not ttl else ttl
@@ -48,10 +70,18 @@ class Confirmation:
         if func_send:
             # TODO: stop token value from leaking into celery logs
             func_send(to=id, template=message, token=token, type=contact.type)
-        return dict(token=token, id=id), 200
+        return dict(token=token, id=id, uid=contact.uid), 200
 
     def confirm(self, token):
         """Confirm a contact if token is still valid
+
+        Args:
+          token (str): the token generated previously
+
+        Returns:
+          tuple:
+            first item is dict with fields as defined in SessionManager.create,
+            second item is the http response code
         """
 
         serializer = URLSafeTimedSerializer(self.token_secret)
@@ -78,9 +108,7 @@ class Confirmation:
             dict(referrer_id=None))
         g.db.commit()
 
-        """
-        Create a login session with permissions to modify a Person/Contact
-        """
+        # Create a login session with permissions to modify a Person/Contact
         logging.info('action=confirm type=%s info=%s' %
                      (contact.type, info))
         return dict(g.session.create(
